@@ -1,5 +1,5 @@
 /* walker.c -- nifty file-tree walker
-   Copyright (C) 1986, 1995-1996, 1999-2000, 2008-2010 Free Software
+   Copyright (C) 1986, 1995-1996, 1999-2000, 2008-2012 Free Software
    Foundation, Inc.
    Written by Greg McGary <gkm@gnu.ai.mit.edu>
 
@@ -20,7 +20,6 @@
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <stddef.h>
 #include <unistd.h>
 #include <string.h>
 #include <fnmatch.h>
@@ -60,7 +59,6 @@ static struct file_link *make_link_from_string (char const *name,
 static int lang_wanted (char const *lang_name);
 static char **append_strings_to_vector (char **vector_0, char *string,
 					char const *delimiter_class);
-static int vector_length (char **vector);
 static int string_in_vector (char const *string, char **vector);
 static int same_as_dot (char const *cwd);
 static struct file_link const
@@ -366,7 +364,7 @@ find_alias_link (struct file_link *flink, struct stat *stp)
    directory.  PATH_MAX is considered an infinite distance (e.g.,
    there are no symlinks between `flink' and the root).  */
 
-static int
+static int _GL_ATTRIBUTE_PURE
 symlink_ancestry (struct file_link *flink)
 {
   int ancestry = 0;
@@ -502,11 +500,20 @@ include_languages (char *lang_names)
 void
 exclude_languages (char *lang_names)
 {
-  if (langs_excluded)
+  if (langs_included)
     error (EXIT_FAILURE, 0, _("can't mix --include and --exclude options"));
   langs_excluded = append_strings_to_vector (langs_excluded, lang_names, white_space);
 }
 
+static int _GL_ATTRIBUTE_PURE
+vector_length (char **vector)
+{
+  int length = 0;
+  while (*vector++)
+    length++;
+  return length;
+}
+
 static char **
 append_strings_to_vector (char **vector_0, char *string,
 			  char const *delimiter_class)
@@ -528,16 +535,7 @@ append_strings_to_vector (char **vector_0, char *string,
   return xnrealloc (vector_0, vector - vector_0, sizeof *vector_0);
 }
 
-static int
-vector_length (char **vector)
-{
-  int length = 0;
-  while (*vector++)
-    length++;
-  return length;
-}
-
-static int
+static int _GL_ATTRIBUTE_PURE
 string_in_vector (char const *string, char **vector)
 {
   while (*vector)
@@ -1053,7 +1051,7 @@ member_file_qsort_compare (void const *x, void const *y)
 /****************************************************************************/
 /* Hash stuff for `struct file_link'.  */
 
-static unsigned long
+static unsigned long _GL_ATTRIBUTE_PURE
 file_link_hash_1 (void const *key)
 {
   unsigned long result = 0;
@@ -1064,7 +1062,7 @@ file_link_hash_1 (void const *key)
   return result;
 }
 
-static unsigned long
+static unsigned long _GL_ATTRIBUTE_PURE
 file_link_hash_2 (void const *key)
 {
   unsigned long result = 0;
@@ -1075,7 +1073,7 @@ file_link_hash_2 (void const *key)
   return result;
 }
 
-static int
+static int _GL_ATTRIBUTE_PURE
 file_link_hash_compare (void const *x, void const *y)
 {
   int result;
