@@ -1,5 +1,5 @@
 /* lid.c -- primary query interface for mkid database
-   Copyright (C) 1986, 1995-1996, 1999-2000, 2007-2010 Free Software
+   Copyright (C) 1986, 1995-1996, 1999-2000, 2007-2012 Free Software
    Foundation, Inc.
    Written by Greg McGary <gkm@gnu.ai.mit.edu>
 
@@ -135,16 +135,10 @@ static void tree8_to_bits_1 (unsigned char **bits_vec,
 static struct file_link **tree8_to_flinkv (unsigned char const *hits_tree8);
 static struct file_link **bits_to_flinkv (unsigned char const *bits_vec);
 
-#if HAVE_TERMIOS_H || HAVE_TERMIO_H || HAVE_SGTTY_H
-#endif
-
 static void savetty (void);
 static void restoretty (void);
 static void chartty (void);
 
-#if HAVE_TERMIOS_H || HAVE_TERMIO_H || HAVE_SGTTY_H
-#endif
-
 #define	TOLOWER(c)	(isupper (c) ? tolower (c) : (c))
 #define IS_ALNUM(c)	(isalnum (c) || (c) == '_')
 
@@ -167,7 +161,7 @@ static int show_version;
 
 /* Which radixes do we want? */
 
-static int radix_flag = radix_all;
+static int radix_flag = 0;
 
 /* If nonzero, ignore differences in alphabetic case while matching.  */
 
@@ -417,6 +411,8 @@ main (int argc, char **argv)
   if (show_help)
     help_me ();
 
+  if (radix_flag == 0)
+    radix_flag = radix_all;
   if (separator_style == ss_contextual)
     {
       if (isatty (STDOUT_FILENO))
@@ -814,7 +810,7 @@ report_nothing (char const *name, struct file_link **flinkv ATTRIBUTE_UNUSED)
     puts (name);
 }
 
-static int
+static int _GL_ATTRIBUTE_PURE
 vector_cardinality (void *vector)
 {
   void **v = (void **) vector;
@@ -1117,6 +1113,8 @@ query_literal_substring (char const *arg, report_func_t report_func)
 static void
 parse_frequency_arg (char const *arg)
 {
+  int range = 0;
+
   if (strnequ (arg, "..", 2))
     frequency_low = 1;
   else
@@ -1124,12 +1122,15 @@ parse_frequency_arg (char const *arg)
       frequency_low = atoi (arg);
       while (isdigit (*arg))
 	arg++;
-      if (strnequ (arg, "..", 2))
-	arg += 2;
+    }
+  if (strnequ (arg, "..", 2))
+    {
+      range = 1;
+      arg += 2;
     }
   if (*arg)
     frequency_high = atoi (arg);
-  else if (strnequ (&arg[-1], "..", 2))
+  else if (range)
     frequency_high = USHRT_MAX;
   else
     frequency_high = frequency_low;
@@ -1238,7 +1239,7 @@ query_binary_search (char const *token_0)
 
 /* Are there any regexp meta-characters in name?? */
 
-static int
+static int _GL_ATTRIBUTE_PURE
 is_regexp (char *name)
 {
   int backslash = 0;
@@ -1283,7 +1284,7 @@ has_right_delimiter (char const *pattern)
 
 /* Does `name' occur in `line' delimited by non-alphanumerics?? */
 
-static int
+static int _GL_ATTRIBUTE_PURE
 word_match (char const *name_0, char const *line)
 {
   char const *name = name_0;
@@ -1318,7 +1319,7 @@ word_match (char const *name_0, char const *line)
    apply.  In particular, it is impossible to determine the radix of
    0, so return all possibilities.  */
 
-static int
+static int _GL_ATTRIBUTE_PURE
 get_radix (char const *str)
 {
   if (!isdigit (*str))
@@ -1373,7 +1374,7 @@ stoi (char const *str)
 
 /* Convert an ascii octal number to an integer. */
 
-static int
+static int _GL_ATTRIBUTE_PURE
 otoi (char const *str)
 {
   int n = 0;
@@ -1390,7 +1391,7 @@ otoi (char const *str)
 
 /* Convert an ascii decimal number to an integer. */
 
-static int
+static int _GL_ATTRIBUTE_PURE
 dtoi (char const *str)
 {
   int n = 0;
@@ -1407,7 +1408,7 @@ dtoi (char const *str)
 
 /* Convert an ascii hex number to an integer. */
 
-static int
+static int _GL_ATTRIBUTE_PURE
 xtoi (char const *str)
 {
   int n = 0;
