@@ -1,5 +1,6 @@
 /* mkid.c -- build an identifer database
-   Copyright (C) 1986, 1995-1996, 1999, 2007-2009 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1995-1996, 1999, 2007-2012 Free Software Foundation,
+   Inc.
    Written by Greg McGary <gkm@gnu.ai.mit.edu>
 
    This program is free software; you can redistribute it and/or modify
@@ -22,7 +23,6 @@
 #include <stdio.h>
 #include <errno.h>
 #include <getopt.h>
-#include <stddef.h>
 #include <unistd.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -86,7 +86,7 @@ static int count_vec_size (struct summary *summary,
 			   unsigned char const *tail_hits);
 static int count_buf_size (struct summary *summary,
 			   unsigned char const *tail_hits);
-static void assert_hits (struct summary* summary);
+static int check_hits (struct summary* summary) _GL_ATTRIBUTE_PURE;
 static void write_hits (FILE *fp, struct summary *summary,
 			unsigned char const *tail_hits);
 static void sign_token (struct token *token);
@@ -443,7 +443,7 @@ main (int argc, char **argv)
 
 /* Return the integer ceiling of the base-8 logarithm of N.  */
 
-static int
+static int _GL_ATTRIBUTE_CONST
 ceil_log_8 (unsigned long n)
 {
   int log_8 = 0;
@@ -459,7 +459,7 @@ ceil_log_8 (unsigned long n)
 
 /* Return the integer ceiling of the base-2 logarithm of N.  */
 
-static int
+static int _GL_ATTRIBUTE_CONST
 ceil_log_2 (unsigned long n)
 {
   int log_2 = 0;
@@ -709,14 +709,20 @@ write_id_file (struct idhead *idhp)
   /* write out the list of pathnames */
 
   fseek (idhp->idh_FILE, sizeof_idhead (), 0);
-  idhp->idh_flinks_offset = ftell (idhp->idh_FILE);
+  off_t off = ftello (idhp->idh_FILE);
+  if (UINT32_MAX < off)
+    error (EXIT_FAILURE, 0, _("internal limitation: offset of 2^32 or larger"));
+  idhp->idh_flinks_offset = off;
   serialize_file_links (idhp);
 
   /* write out the list of identifiers */
 
   putc ('\0', idhp->idh_FILE);
   putc ('\0', idhp->idh_FILE);
-  idhp->idh_tokens_offset = ftell (idhp->idh_FILE);
+  off = ftello (idhp->idh_FILE);
+  if (UINT32_MAX < off)
+    error (EXIT_FAILURE, 0, _("internal limitation: offset of 2^32 or larger"));
+  idhp->idh_tokens_offset = off;
 
   for (i = 0; i < token_table.ht_fill; i++, tokens++)
     {
@@ -758,9 +764,12 @@ write_id_file (struct idhead *idhp)
       putc ('\0', idhp->idh_FILE);
       putc ('\0', idhp->idh_FILE);
     }
-  assert_hits (summary_root);
+  assert (check_hits (summary_root) == 0);
   idhp->idh_tokens = token_table.ht_fill;
-  output_length = ftell (idhp->idh_FILE);
+  off = ftello (idhp->idh_FILE);
+  if (UINT32_MAX < off)
+    error (EXIT_FAILURE, 0, _("internal limitation: offset of 2^32 or larger"));
+  output_length = off;
   idhp->idh_end_offset = output_length - 2;
   idhp->idh_buf_size = max_buf_size;
   idhp->idh_vec_size = max_vec_size;
@@ -773,26 +782,26 @@ write_id_file (struct idhead *idhp)
 /* Define primary and secondary hash and comparison functions for the
    token table.  */
 
-static unsigned long
+static unsigned long _GL_ATTRIBUTE_PURE
 token_hash_1 (void const *key)
 {
   return_STRING_HASH_1 (TOKEN_NAME ((struct token const *) key));
 }
 
-static unsigned long
+static unsigned long _GL_ATTRIBUTE_PURE
 token_hash_2 (void const *key)
 {
   return_STRING_HASH_2 (TOKEN_NAME ((struct token const *) key));
 }
 
-static int
+static int _GL_ATTRIBUTE_PURE
 token_hash_cmp (void const *x, void const *y)
 {
   return_STRING_COMPARE (TOKEN_NAME ((struct token const *) x),
 			 TOKEN_NAME ((struct token const *) y));
 }
 
-static int
+static int _GL_ATTRIBUTE_PURE
 token_qsort_cmp (void const *x, void const *y)
 {
   return_STRING_COMPARE (TOKEN_NAME (*(struct token const *const *) x),
@@ -929,7 +938,7 @@ make_sibling_summary (struct summary *summary)
   return summary;
 }
 
-static int
+static int _GL_ATTRIBUTE_PURE
 count_vec_size (struct summary *summary, unsigned char const *tail_hits)
 {
   struct summary **kids;
@@ -953,7 +962,7 @@ count_vec_size (struct summary *summary, unsigned char const *tail_hits)
     }
 }
 
-static int
+static int _GL_ATTRIBUTE_PURE
 count_buf_size (struct summary *summary, unsigned char const *tail_hits)
 {
   struct summary **kids;
@@ -974,19 +983,25 @@ count_buf_size (struct summary *summary, unsigned char const *tail_hits)
     }
 }
 
-static void
-assert_hits (struct summary* summary)
+/* Sanity-check hit counts.  Return nonzero if there's a problem.
+   Otherwise, return 0.  */
+static int
+check_hits (struct summary* summary)
 {
   struct summary **kids = summary->sum_kids;
   struct summary **end = &kids[8];
 
-  assert (summary->sum_hits == NULL || *summary->sum_hits == 0);
+  if ( ! (summary->sum_hits == NULL || *summary->sum_hits == 0))
+    return 1;
 
   if (end[-1] == 0)
     while (*--end == 0)
       ;
   while (kids < end)
-    assert_hits (*kids++);
+    if (check_hits (*kids++))
+      return 1;
+
+  return 0;
 }
 
 static void
